############################# PSQF 6272 Example 3 using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs using lme or gls

if (!require("psychometric")) install.packages("psychometric")
library(psychometric) # To ICCs using lme or gls

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 3             #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example3/"
filename = "Example3_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "HSB4"
Example3 = read_excel(paste0(filesave,filename), sheet="HSB4") 
# Convert to data frame to use in analysis
Example3 = as.data.frame(Example3)

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example3_R_Output.txt"))

# Filter to only cases complete on all variables to be used below (before cluster means)
Example3 = Example3[complete.cases(Example3[ , c("math","ses","nonwhite")]),]

# Compute cluster means for level-1 variables using Jonathan's function
Example3 = addUnitMeans(data=Example3, unitVariable="schoolID",
                        meanVariables=c("math","ses","nonwhite"), 
                        newNames=c("CMmath","CMses","CMnon"))

print("R Descriptive Statistics for Cluster Mean Level-2 Variables")
describe(x=Example3[ , c("CMmath","CMses","CMnon")])

print("R Pearson Correlation Matrix with P-values using rcorr from Hmisc package")
print("Must convert data frame to matrix to use rcorr")
rcorr(x=as.matrix(Example3[,c("CMmath","CMses","CMnon")]), type="pearson")

print("R Descriptive Statistics for Original Level-1 Variables")
describe(x=Example3[ , c("math","ses","nonwhite")])

# Center L2 predictor at constant so 0 is meaningful
Example3$CMnon30 = Example3$CMnon - .30
# 0 is otherwise already meaningful in other predictors

# Center L1 predictors at cluster means
Example3$WCses = Example3$ses - Example3$CMses
Example3$WCnon = Example3$nonwhite - Example3$CMnon
# Center outcome at cluster mean for correlations
Example3$WCmath = Example3$math - Example3$CMmath;

print("R Descriptive Statistics for Within-Cluster Level-1 Variables")
describe(x=Example3[ , c("WCmath","WCses","WCnon")])

print("R Pearson Correlation Matrix with P-values using rcorr from Hmisc package")
print("Must convert data frame to matrix to use rcorr")
rcorr(x=as.matrix(Example3[,c("WCmath","WCses","WCnon")]), type="pearson")


################################################################################
#####                 BEGIN MODELS FOR EXAMPLE 3                           #####
################################################################################

# Empty models

print("R Model 1: Empty Means, Random Intercept for Math")
Model1 = lmer(data=Example3, REML=TRUE, formula=math~1+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1, chkREML=FALSE); summary(Model1, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(Model1); ranova(Model1)

txtComment(" ") # insert blank space in output


print("R: Empty Means, Random Intercept for SES")
EmptySES = lmer(data=Example3, REML=TRUE, formula=ses~1+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(EmptySES, chkREML=FALSE); summary(EmptySES, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(EmptySES); ranova(EmptySES)

txtComment(" ") # insert blank space in output


print("R: Empty Means, Random Intercept for Binary Nonwhite")
EmptyNon = glmer(data=Example3, family=binomial(link="logit"), nonwhite~1+(1|schoolID))
summary(EmptyNon) # deviance = -2LL already

# Compute ICC using pi^2/3 = 3.29 as L1 residual variance 
# (sub-object theta holds random intercept variance)
EmptyNon@theta^2/(EmptyNon@theta^2+(pi^2/3)) 
icc(EmptyNon) # easier way

print("R: Single-level empty model predicting observed free/reduced lunch ignoring school")
SingleNon = glm(data=Example3, family=binomial(link="logit"), formula=nonwhite~1)
print("Likelihood Ratio Test for Addition of Random Intercept Variance")
DevTest=-2*(logLik(SingleNon)-logLik(EmptyNon))
Pvalue=pchisq((DevTest), df=1, lower.tail=FALSE)
print("Test Statistic and P-values for DF=1") 
DevTest; Pvalue


txtComment(" ") # insert blank space in output


##################################################################################

# Add SES predictors

print("R Model 2a: Add L1 Cluster-MC Student SES")
Model2a = lmer(data=Example3, REML=TRUE, formula=math~1+WCses+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2a, chkREML=FALSE); summary(Model2a, ddf="Satterthwaite")

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model2a)

txtComment(" ") # insert blank space in output


print("R Model 2b: Add L1 Cluster-MC Student SES")
Model2b = lmer(data=Example3, REML=TRUE, formula=math~1+WCses+CMses+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2b, chkREML=FALSE); summary(Model2b, ddf="Satterthwaite")
print("SES L2 Contextual Slope"); contest1D(Model2b, L=c(0,-1,1))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model2b)

txtComment(" ") # insert blank space in output


print("R Model 3a: Use Smushed Main Effect of L1 Student SES Instead")
Model3a = lmer(data=Example3, REML=TRUE, formula=math~1+ses+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3a, chkREML=FALSE); summary(Model3a, ddf="Satterthwaite")

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model3a)

txtComment(" ") # insert blank space in output


print("R Model 3b: L1 Grand-MC Student SES + L2 Cluster Mean SES")
Model3b = lmer(data=Example3, REML=TRUE, formula=math~1+ses+CMses+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3b, chkREML=FALSE); summary(Model3b, ddf="Satterthwaite")
print("SES L2 Between Slope"); contest1D(Model3b, L=c(0,1,1))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model3b)

txtComment(" ") # insert blank space in output


######################################################################################

# Add nonwhite predictors

print("R Model 4a: Add L1 Binary Student Nonwhite")
Model4a = lmer(data=Example3, REML=TRUE, formula=math~1+ses+CMses+nonwhite+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4a, chkREML=FALSE); summary(Model4a, ddf="Satterthwaite")

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model4a)

txtComment(" ") # insert blank space in output


print("R Model 4b: L1 Binary Student Nonwhite + L2 Cluster Mean Nonwhite")
Model4b = lmer(data=Example3, REML=TRUE, 
               formula=math~1+ses+CMses+nonwhite+CMnon30+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4b, chkREML=FALSE); summary(Model4b, ddf="Satterthwaite")
print("SES L2 Between Slope");      contest1D(Model4b, L=c(0,1,1,0,0))
print("Nonwhite L2 Between Slope"); contest1D(Model4b, L=c(0,0,0,1,1))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model4b)

txtComment(" ") # insert blank space in output


print("R Model 4c: Use Cluster-MC Versions of L1 Predictors Instead")
Model4c = lmer(data=Example3, REML=TRUE, 
               formula=math~1+WCses+CMses+WCnon+CMnon30+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4c, chkREML=FALSE); summary(Model4c, ddf="Satterthwaite")
print("SES L2 Contextual Slope");      contest1D(Model4c, L=c(0,-1,1,0,0))
print("Nonwhite L2 Contextual Slope"); contest1D(Model4c, L=c(0,0,0,-1,1))
print("Nonwhite L2 Between Slope per 10%");    contest1D(Model4c, L=c(0,0,0,0,1/10))
print("Nonwhite L2 Contextual Slope per 10%"); contest1D(Model4c, L=c(0,0,0,-1/10,1/10))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model4c)

print("Total-R2 for SES relative to empty model using Jonathan's function")
totalRSquaredinator(model=Model2b, dvName="math", data=Example3)

print("Total-R2 for SES+Nonwhite relative to empty model using Jonathan's function")
totalRSquaredinator(model=Model4c, dvName="math", data=Example3)

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
txtStop()



